/*
 *  phidgetnand.c
 *
 *  Based on nandwrite, flash_eraseall
 *
 *  Copyright (C) 2000 Steven J. Hill (sjhill@realitydiluted.com)
 *		  2003 Thomas Gleixner (tglx@linutronix.de)
 *		  2008 Phdigets Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Overview:
 *   This utility writes a binary image directly to a NAND flash
 *   chip or NAND chips contained in DoC devices. This is the
 *   "inverse operation" of nanddump.
 *
 * tglx: Major rewrite to handle bad blocks, write data with or without ECC
 *	 write oob data only on request
 *
 * Bug/ToDo:
 */

#include "phidgetnand.h"

int boot = 0;

void display_help (void)
{
	printf("Usage: phidgetnand write|erase [OPTION] MTD_DEVICE [INPUTFILE]\n"
			"Writes to or erases the specified MTD device.\n"
			"\n"
			"  -r, --reboot	  reboot when done\n"
		    "  -q, --quiet    don't display progress messages\n"
		    "      --help     display this help and exit\n"
		    "      --version  output version information and exit\n"
			"\n"
			" Write options:\n"
			"  -a, --autoplace	Use auto oob layout\n"
			"  -e, --erase		erase whole partition before writing\n"
			"  -j, --jffs2		force jffs2 oob layout (legacy support)\n"
			"  -y, --yaffs		force yaffs oob layout (legacy support)\n"
			"  -f, --forcelegacy     force legacy support on autoplacement enabled mtd device\n"
			"  -n, --noecc		write without ecc\n"
			"  -o, --oob		image contains oob data\n"
			"  -s addr, --start=addr set start address (default is 0)\n"
			"  -p, --pad             pad to page size\n"
			"  -b, --blockalign=1|2|4 set multiple of eraseblocks to align to\n"
		    "\n"
		    " Erase options:\n"
		    "  -j, --jffs2    format the device for jffs2\n");
	exit(0);
}

void display_version (void)
{
	printf(PROGRAM " " VERSION "\n"
			"\n"
			"Copyright (C) 2008 Phidgets Inc. \n"
			"\n"
			PROGRAM " comes with NO WARRANTY\n"
			"to the extent permitted by law.\n"
			"\n"
			"You may redistribute copies of " PROGRAM "\n"
			"under the terms of the GNU General Public Licence.\n"
			"See the file `COPYING' for more information.\n");
	exit(0);
}

static void do_reboot(void)
{
	fprintf(stderr, "Rebooting ...\n");
	fflush(stderr);
	
	/* try regular reboot method first */
	system("/sbin/reboot");
	sleep(2);
	
	/* if we're still alive at this point, force the kernel to reboot */
	syscall(SYS_reboot,LINUX_REBOOT_MAGIC1,LINUX_REBOOT_MAGIC2,LINUX_REBOOT_CMD_RESTART,NULL);
}

/*
 * Main program
 */
int main(int argc, char **argv)
{	
	if(argc < 3)
		display_help();
	
	optind = 2;
	
	//if any of these error, they will exit(1)
	if(!strcmp(argv[1], "write"))
		nand_write(argc, argv);
	else if(!strcmp(argv[1], "erase"))
		nand_erase(argc, argv);
	else
		display_help();
	
	if (boot)
	{
		printf("Rebooting...\n"); fflush(stdout);
		do_reboot();
	}
	
	return 0;
}
