#!/bin/sh
#
# Default handlers for config files
#

. /etc/config.sh
. /usr/lib/webif/functions.sh

echo "Applying changes..."

HANDLERS_config='
	webcam) reload_webcam;;
	system) reload_system;;
	network) reload_network;;
	phidgetwebservice21) reload_webservice;;
	wireless_network) reload_wireless_network;;
'

mkdir -p "/tmp/.webif"
_pushed_dir=$(pwd)
cd "/tmp/.webif"

# edited-files/*		user edited files - stored with directory tree in-tact
#for edited_file in $(find "/tmp/.webif/edited-files/" -type f 2>&-); do
find "/tmp/.webif/edited-files/" -type f 2>&- | while read -r edited_file
do
	target_file=$(echo "$edited_file" | sed s/'\/tmp\/.webif\/edited-files'//g)
	echo "Processing $target_file"
	fix_symlink_hack "$target_file"
	if tr -d '\r' <"$edited_file" >"$target_file"; then
		rm "$edited_file" 2>&-
	else
		echo "Critical Error : Could not replace $target_file. Media full?"
	fi
done
# leave if some files not applied
rm -r "/tmp/.webif/edited-files" 2>&-

# merges the tmp into the real
update_config() {
	# this reads in lines, so we can have spaces in config values
	while read -r config
	do
		key="${config%%=*}"
		val="${config#*=}"
		#echo "Key = $key, Val = $val"
		grep "^$key=" $SBC_CONFIGDIR/$1.conf >&- 2>&- && {
			grep -v "^$key=" $SBC_CONFIGDIR/$1.conf > $SBC_CONFIGDIR/$1.conf.new 2>&- 
			mv $SBC_CONFIGDIR/$1.conf.new $SBC_CONFIGDIR/$1.conf 2>&- >&-
		}
		echo "$key=$val" >> $SBC_CONFIGDIR/$1.conf
	done < /tmp/.config/$1.conf
	#IFS=$OLDIFS
}

reload_wireless_network() {
	echo 'Reloading wireless networking settings ...'
	
	. /etc/config.sh

	create_interfaces || echo "WARNING: problem creating interfaces file"
	do_dns

	/sbin/wpa_action wlan0 CONNECTED

	/etc/init.d/S80mdnsresponder reload
	/sbin/ntpd_startstop restart
}

reload_network() {
	echo 'Reloading networking settings ...'
	
	. /etc/config.sh

	create_interfaces || echo "WARNING: problem creating interfaces file"
	do_dns

	ifdown -a
	ifup -a

	/etc/init.d/S50dropbear start
	/etc/init.d/S80mdnsresponder reload
	/sbin/ntpd_startstop restart
}

reload_webcam() {
	echo 'Reloading webcam settings ...'

	/sbin/webcam restart /dev/video0
}

reload_system() {
	echo 'Reloading system settings ...'
	
	. /etc/config.sh

	[ -z $SBC_HOSTNAME ] || (echo $SBC_HOSTNAME > /etc/hostname; /bin/hostname -F /etc/hostname)

	echo $SBC_TZ > /etc/TZ
	
	# restart to refresh the hostname change
	/etc/init.d/S80mdnsresponder restart
}

reload_webservice() {
	echo 'Reloading Phidget Webservice settings ...'

	/etc/init.d/S90phidgetwebservice21 restart
}

mkdir -p "/tmp/.config"
cd /tmp/.config

# *.conf files

for config in $(ls *.conf 2>&-); do
	name=${config%.conf}
	echo "Processing config file: $name"
	update_config $name
	eval 'case "$name" in
		'"$HANDLERS_config"'
	esac'
done

sleep 1
rm -f *.conf
