libdir=/usr/lib/webif
wwwdir=/www
cgidir=/www/cgi-bin/webif
rootdir=/cgi-bin/webif
indexpage=index.sh
. /usr/lib/webif/functions.sh

categories() {
	grep '##WEBIF:' $cgidir/.categories $cgidir/*.sh 2>/dev/null | \
		awk -v "selected=$1" \
			-v "rootdir=$rootdir" \
			-v "indexpage=$indexpage" \
			-f /usr/lib/webif/categories.awk -
}

subcategories() {
	grep -H "##WEBIF:name:$1:" $cgidir/*.sh 2>/dev/null | \
		sed -e 's,^.*/\([a-zA-Z0-9_\.\-]*\):\(.*\)$,\2:\1,' | \
		sort -n | \
		awk -v "selected=$2" \
			-v "rootdir=$rootdir" \
			-f /usr/lib/webif/subcategories.awk -
}

update_changes() {
	CHANGES=$(($( (cat /tmp/.config/*.conf) 2>&- | wc -l)))
	EDITED_FILES=$(find "/tmp/.webif/edited-files" -type f 2>&- | wc -l)
	CHANGES=$(($CHANGES + $EDITED_FILES))
}

#######################################################
# header
#######################################################
header() {
	update_changes
	_changes=${CHANGES#0}
	_changes=${_changes:+(${_changes})}
	# saved=0 if we're on the config.sh
	if equal "$SAVED" "0"; then
		SAVED=
	else
		empty "$ERROR" && {
			if ! equal "$_changes" ""; then
				SAVED="${SAVED:+Settings saved}"
				empty "$SAVED" && {
					SAVED="<font color="#E7292C">Pending settings changes exist.</font>"
				}
				MESSAGE="<font color="#E7292C">Don't forget to commit, or these changes will be lost.</font>${MESSAGE:+<br /><br />$MESSAGE}"
			else
				SAVED="${SAVED:+<font color="#E7292C">No changes were made</font>}"
			fi

		} || {
			FORM_submit="";
			SAVED="<font color="#E7292C">Error(s)</font>"
		}
	fi

	_category="$1"
	_uptime="$(uptime)"
	_loadavg="${_uptime#*load average: }"
	_uptime="${_uptime#*up }"
	_uptime="${_uptime%%,*}"
	_hostname=$(cat /proc/sys/kernel/hostname)
	_version=$( grep "version" /etc/phidgetsbc_version | awk '{print $2}' )
	_board_version="$(cat /proc/cpuinfo | awk '/^Revision/ {print ("0x"$3)+0}')"
	# progress bar causes some error - somehow config.sh is run twice, figure this out before enabling
	_use_progressbar="0"
	_head="${3:+<h2>$3</h2>}"
	_form="${5:+<form enctype=\"multipart/form-data\" action=\"$5\" method=\"post\"><input type=\"hidden\" name=\"submit\" value=\"1\" />}"
	_savebutton="${5:+<div class=\"page-save\"><input type=\"submit\" name=\"action\" value=\"Save Changes\" /></div>}"
	_categories=$(categories $1)
	_subcategories=${2:+$(subcategories "$1" "$2")}
	_pagename="${2:+$2 - }"

	empty "$REMOTE_USER" && neq "${SCRIPT_NAME#/cgi-bin/}" "webif.sh" && grep 'root:!' /etc/passwd >&- 2>&- && {
		_nopasswd=1
		_form=""
		_savebutton=""
	}

	cat <<EOF
Content-Type: text/html; charset=UTF-8
Pragma: no-cache

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
<head>
    	<title>PhidgetSBC Configuration</title>
	<link rel="stylesheet" type="text/css" href="/css/waitbox.css" media="screen" />
	<link rel="stylesheet" type="text/css" href="/css/webif.css" />
	<!--[if lt IE 7.]>
		<script defer type="text/javascript" src="/js/pngfix.js"></script>
	<![endif]-->
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
	<meta http-equiv="expires" content="-1" />
$header_inject_head</head>

<body>$header_inject_body

<div id="container">
	<div id="header">
		<br />
		<h1 valign="center">PhidgetSBC Configuration </h1>
		<div id="header-picture"><img src="/images/phidgets/1070_0_Thumbnail.png" valign="center" /></div>
		<div id="short-status">
			<h3><strong>Status:</strong></h3>
			<ul>
				<li><strong>Host Name:</strong> $_hostname</li>
				<li><strong>Uptime:</strong> $_uptime</li>
				<li><strong>Load:</strong> $_loadavg</li>
				<li><strong>Version:</strong> $_board_version, $_version</li>
			</ul>
		</div>
	</div>

	<div id="mainmenu">
	$_categories
	</div>

	<div id="submenu">
	$_subcategories
	</div>

EOF

cat <<EOF

	$_form
	<div id="content">
EOF
		# These can live alone
		! empty "$SAVED" || ! empty "$ERROR" || ! empty "$MESSAGE" && {
			echo "<div id=\"save_messages\">"
			! empty "$SAVED" && {
				echo "<h3>$SAVED</h3>"
			}
			! empty "$ERROR" && {
				echo "<p>$ERROR</p>"
			}
			! empty "$MESSAGE" && {
				echo "<p>$MESSAGE</p>"
			}
			echo "</div>"
		}
		
	echo "$_head"

	empty "$REMOTE_USER" && neq "${SCRIPT_NAME#/cgi-bin/}" "webif.sh" && {
		! empty "$FORM_passwd1$FORM_passwd2" && {
			equal "$FORM_passwd1" "$FORM_passwd2" && {
				validate <<EOF
password|FORM_passwd1|Password|required min=1 max=63|$FORM_passwd1
EOF
				empty "$ERROR" && {
					echo '<pre>'
					(
						echo "$FORM_passwd1"
						sleep 1
						echo "$FORM_passwd2"
					) | passwd root 2>&1
					(
						echo "$FORM_passwd1"
						sleep 1
						echo "$FORM_passwd2"
					) | passwd user 2>&1
					echo '</pre>'
					echo "<meta http-equiv=\"refresh\" content=\"2; URL=$SCRIPT_NAME\" />"
					footer
					exit
				} || {
				echo "<div id=\"save_messages\">$ERROR</div>"
				}
			} || {
				echo "<div id=\"save_messages\">The entered passwords do not match!</div>"
			}
		}
		equal "$_nopasswd" 1 && {
			cat <<EOF

<h3>Warning: You haven't set a password for the Web interface and SSH access.<br />Please enter one now (the user name in your browser will be 'root').</h3>
<br />
EOF
			empty "$NOINPUT" && cat <<EOF
<form enctype="multipart/form-data" action="$SCRIPT_NAME" method="POST">
<table>
	<tr>
		<td>New Password:</td>
		<td><input type="password" name="passwd1" /></td>
	</tr>
	<tr>
		<td>Confirm Password:</td>
		<td><input type="password" name="passwd2" /></td>
	</tr>
	<tr>
		<td colspan="2"><br /><input type="submit" name="action" value="Set" /></td>
	</tr>
</table>
</form>
EOF
			footer
			exit
		}
	}
}
# end of header


#######################################################
# footer
#######################################################
footer() {
	update_changes
	_changes=${CHANGES#0}
	_changes=${_changes:+(${_changes})}
	_endform=${_savebutton:+</form>}

	cat <<EOF

	</div> <!-- End #content -->

	<br />
	<fieldset id="save">
	<legend><strong>Proceed Changes</strong></legend>
	$_savebutton
EOF

	equal "$_use_progressbar" "1" && {
		echo '<script type="text/javascript" src="/js/waitbox.js"></script>'
	}

	if ! equal "$_changes" ""; then
	cat <<EOF
		<ul class="apply">
			<li><a href="config.sh?mode=save&amp;cat=$_category&amp;prev=$SCRIPT_NAME" rel="lightbox">Commit Changes &laquo;</a></li>
			<li><a href="config.sh?mode=clear&amp;cat=$_category&amp;prev=$SCRIPT_NAME">Clear Changes &laquo;</a></li>
			<li><a href="config.sh?mode=review&amp;cat=$_category&amp;prev=$SCRIPT_NAME">Review Changes $_changes &laquo;</a></li>
		</ul>
EOF
	else
		echo "<ul class=\"apply\"><li>&nbsp;</li><li>&nbsp;</li><li>&nbsp;</li></ul>"
	fi
	cat <<EOF
	</fieldset>
	$_endform
	<hr />
	<div id="footer">
		<h3>Phidgets Inc.</h3>
	</div>
</div> <!-- End #container -->
</body>
</html>
EOF
}

#######################################################

display_form() {
	if empty "$1"; then
		awk -F'|' -f /usr/lib/webif/common.awk -f /usr/lib/webif/form.awk
	else
		echo "$1" | awk -F'|' -f /usr/lib/webif/common.awk -f /usr/lib/webif/form.awk
	fi
}

list_remove() {
	echo "$1 " | awk '
BEGIN {
	RS=" "
	FS=":"
}
($0 !~ /^'"$2"'/) && ($0 != "") {
	printf " " $0
	first = 0
}'
}

handle_list() {
	# $1 - remove
	# $2 - add
	# $3 - submit
	# $4 - validate
	
	empty "$1" || {
		LISTVAL="$(list_remove "$LISTVAL" "$1") "
		LISTVAL="${LISTVAL# }"
		LISTVAL="${LISTVAL%% }"
		_changed=1
	}
	
	empty "$3" || {
		validate "${4:-none}|$2" && {
			if empty "$LISTVAL"; then
				LISTVAL="$2"
			else
				LISTVAL="$LISTVAL $2"
			fi
			_changed=1
		}
	}

	LISTVAL="${LISTVAL# }"
	LISTVAL="${LISTVAL%% }"
	LISTVAL="${LISTVAL:- }"

	if empty "$_changed"; then
		return 255
	else
		return 0
	fi
}

