#!/usr/bin/webif-page
<?
. "/usr/lib/webif/webif.sh"

load_settings system

###################################################################
# system configuration page
#
# Description:
#	Configures general system settings.
#

#####################################################################
# initialize forms
if empty "$FORM_submit"; then
	# initialize all defaults
	time_zone_part=$posixtz
	time_zoneinfo_part=$zoneinfo

	time_zone_part="${time_zone_part:-"UTC+0"}"
	time_zoneinfo_part="${time_zoneinfo_part:-"-"}"

	FORM_system_timezone="${time_zoneinfo_part}@${time_zone_part}"

	FORM_hostname=$hostname
else
#####################################################################
# save forms
	SAVED=1
	validate <<EOF
hostname|FORM_hostname|Hostname|nodots required max=63 cfg|$FORM_hostname
string|FORM_show_TZ|Zoneinfo String|cfg|$FORM_show_TZ
string|FORM_show_TZ_zone|POSIX TZ String|cfg|$FORM_show_TZ_zone
EOF
	if equal "$?" 0 ; then

		save_setting system hostname "$FORM_hostname"

		save_setting system posixtz "$FORM_show_TZ"
		save_setting system zoneinfo "$FORM_show_TZ_zone"
	fi
fi

#####################################################################
# initialize time zones

TIMEZONE_OPTS=$(
	awk -v timezoneinfo="$FORM_system_timezone" '
		BEGIN {
			FS="	"
			last_group=""
			defined = 0
		}
		/^(#.*)?$/ {next}
		$1 != last_group {
			last_group=$1
			print "optgroup|" $1
		}
		{
			list_timezone = $4 "@" $3
			if (list_timezone == timezoneinfo)
				defined = defined + 1
			print "option|" list_timezone "|" $2 ""
		}
		END {
			if (defined == 0) {
				split(timezoneinfo, oldtz, "@")
				print "optgroup|User Defined"
				if (oldtz[1] == "-") oldtz[1] = "User defined"
				print "option|" timezoneinfo "|" oldtz[1]
			}
		}' < /usr/lib/webif/timezones.csv 2>/dev/null

)

#####################################################################

header "System" "Settings" "System Settings" '' "$SCRIPT_NAME"

#####################################################################

cat <<EOF
<script type="text/javascript" src="/webif.js"></script>
<script type="text/javascript">
<!--
function modechange()
{
	var tz_info = value('system_timezone');
	if ((tz_info=='') || (tz_info==null)){
		set_value('show_TZ', tz_info);
		set_value('show_TZ_zone', tz_info);
	}
	else {
		var tz_split = tz_info.split('@');
		set_value('show_TZ', tz_split[1]);
		set_value('show_TZ_zone', tz_split[0]);
	}
}
-->
</script>
EOF

#######################################################
# Show form
display_form <<EOF
onchange|modechange
start_form|System Settings
field|Hostname
text|hostname|$FORM_hostname
helpitem|Hostname
helptext|The system hostname. This is used for the system's mDNS hostname, as well as the Phidget Webservice default Server ID.
end_form
start_form|Time Settings
field|Timezone
select|system_timezone|$FORM_system_timezone
$TIMEZONE_OPTS
field|Zoneinfo String|view_tz_zone|
string|<input id="show_TZ_zone" type="text" style="width: 96%; height: 1.2em; " name="show_TZ_zone" value="" />
field|POSIX TZ String|view_tz_string|
string|<input id="show_TZ" type="text" style="width: 96%; height: 1.2em; " name="show_TZ" value="" />
helpitem|Timezone
helptext|Set up your time zone according to the nearest city of your region from the predefined list, or enter your own time zone.
helpitem|Zoneinfo String
helptext|Standard zoneinfo names are defined for different areas of the world. Check <a href="http://en.wikipedia.org/wiki/List_of_zoneinfo_timezones">here</a> for a list of zones.
helpitem|POSIX TZ String
helptext|This is a Posix standard for specifying the time zone. The format is:<br /> <font face="courier">stdoffset[dst[offset][,start[/time],end[/time]]]</font><br /> Check <a href="http://www.opengroup.org/onlinepubs/000095399/basedefs/xbd_chap08.html#tag_08_03">here</a>, under 'TZ' for more information.
$NTP
end_form
EOF

footer ?>

<script Language="JavaScript">
<!--
modechange();
-->
</script>

<!--
##WEBIF:name:System:010:Settings
-->
