#!/usr/bin/webif-page --upload-limit=5530
<?
. /usr/lib/webif/webif.sh
. /etc/config.sh

HOME_DIR="/home/user"

###################################################################
# userspace-editor
#
# Description:
#	Filesystem browser/File editor.
#       This file is compatible with both branches and
#       should be synchronized between branches
#
# Author(s) [in order of work date]:
#       unknown
#       Lubos Stanek <lubek@users.berlios.de>
#       Patrick McNeil - Phidgets Inc.
#
# Required components:
#       /usr/lib/webif/common.awk
#       /usr/lib/webif/browser.awk
#       /usr/lib/webif/editor.awk
#

header_inject_head=$(cat <<EOF
<script defer type="text/javascript" src="/webif.js"></script>
<script type="text/javascript">
<!--
function webif_entityDecode(s) {
    var e = document.createElement("div");
    e.innerHTML = s;
    return e.firstChild.nodeValue;
}

var webif_printf=function() {
	var num = arguments.length;
	var output = arguments[0];
	for (var i = 1; i < num; i++) {
		var pattern = "\\\\{" + (i-1) + "\\\\}";
		var re = new RegExp(pattern, "g");
		output = output.replace(re, arguments[i]);
	}
	return output;
}

function confirm_deldir(path,file) {
	if (window.confirm(webif_entityDecode(webif_printf("@TR<<big_warning#WARNING>>!\\n\\nDo you really want to delete the '{0}' directory?", file)))) {
		window.location="$SCRIPT_NAME?path=" + encodeURIComponent(path) + "&delpath=" + encodeURIComponent(file);
	}
}
function confirm_delfile(path,file) {
	if (window.confirm(webif_entityDecode(webif_printf("@TR<<big_warning#WARNING>>!\\n\\n@TR<<system_editor_ask_file_deletition#Do you really want to delete the '{0}/{1}' file>>?", path, file)))) {
		window.location="$SCRIPT_NAME?path=" + encodeURIComponent(path) + "&delfile=" + encodeURIComponent(file);
	}
}

function statusupdate() {
	document.getElementById("form_submit").style.display = "none";

	return true;
}

function fix_uris()
{
	
}

-->
</script>

<style type="text/css">
<!--
#filebrowser table {
	margin-left: 1em;
	margin-right: 1em;
	margin-bottom: 2.2em;
	text-align: left;
	font-size: 0.8em;
	border-style: none;
	border-spacing: 0;
}
#filebrowser td {
	padding-left: 0.1em;
	padding-right: 0.1em;
}
#filebrowser td.number {
	text-align: right;
}
#filebrowser td.leftimage {
	padding-left: 0em;
}
#filebrowser td.image {
	text-align: center;
}
#filebrowser td.rightimage {
	padding-right: 0em;
}
#filebrowser a.tooltip {
	z-index: 3;
}
#filebrowser span.tooltip {
	display: none;
}
#filebrowser a:hover.tooltip {
	position: relative;
}
#filebrowser a:hover.tooltip span.tooltip {
	z-index: 5;
	display: block;
	position: absolute;
	top: 1.1em;
	left: 1.8em;
	width: 20em;
	text-decoration: none;
	padding: 3px;
	border: 1px solid;
	color: Black;
	background-color: White;
}
-->
</style>

EOF
)

############################
# deleters
############################

! empty "$FORM_delpath" && {
	cd / 2>/dev/null
	ERROR=$(rmdir "$FORM_delpath" 2>&1)
	equal "$?" "0" && {
		MESSAGE=$(cat <<EOF
Directory was deleted successfully:<br/>
<strong>$FORM_delpath</strong>
EOF
)
	}
}

! empty "$FORM_delfile" && {
	ERROR=$(rm "$FORM_path/$FORM_delfile" 2>&1)
	equal "$?" "0" && {
		MESSAGE=$(cat <<EOF
File was deleted successfully:<br/>
<strong>$FORM_path/$FORM_delfile</strong>
EOF
)
	}
}

############################
# fix path
############################

FORM_path="${FORM_path:-/}"
ERROR="$ERROR$(cd "$FORM_path" 2>&1)"
cd "$FORM_path" 2>/dev/null
while [ "$?" != "0" ]; do
	FORM_path="${FORM_path%/*}"
	FORM_path="${FORM_path:-/}"
	cd "$FORM_path" 2>/dev/null
done
FORM_path="$(pwd)"

# don't travel outside of SBC_USERAPPS
if [ ! `expr substr $FORM_path 1 ${#HOME_DIR}` = $HOME_DIR ]; then
	FORM_path=$HOME_DIR
	cd $FORM_path
fi

# return to the cgi dir
cd "${SCRIPT_NAME%/*}" 2>/dev/null

############################
# directory was created
############################
if ! equal "$FORM_dirname" ""; then

	SAVED=1
	validate <<EOF
filedir|FORM_dirname|Directory Name|required min=1 max=63|$FORM_dirname
EOF
	if equal "$?" 0 ; then
		SAVED=
		#strip any path info - just want the filename
		filename="${FORM_file_name##*\\}"
		filename="${filename##*/}"
		if [ -e "$FORM_path/$FORM_dirname" ]; then
			ERROR="Error creating directory $FORM_dirname - a file or directory of the same name already exists."
		else
			if mkdir "$FORM_path/$FORM_dirname" && chown user:user "$FORM_path/$FORM_dirname"; then
				MESSAGE="Directory was created successfully:<br/><strong>$FORM_dirname</strong>"
			else
				ERROR="Error creating directory $FORM_dirname"
			fi
		fi
	fi
	
fi

############################
# file was uploaded
############################
if ! equal "$FORM_file" ""; then
	#strip any path info - just want the filename
	filename="${FORM_file_name##*\\\\}"
	filename="${filename##*/}"
	
	validate <<EOF
filedir|filename|File Name|required min=1 max=63|$filename
EOF

	if equal "$?" 0 ; then
		fsize=$(ls -alL "$FORM_file" 2>/dev/null | awk '{ print $5 }')
		freespace_bytes=$(df $SBC_USERSPACE | grep / | awk {'print $4*1024'})
		if [ "$fsize" -gt "$freespace_bytes" ]; then
			ERROR="Error recieving file $filename - not enough free space"
			rm "$FORM_file"
		elif [ -e "$FORM_path/$filename" ]; then
			ERROR="Error recieving file $filename - a file or directory of the same name already exists."
			rm "$FORM_file"
		else
			if mv "$FORM_file" "$FORM_path/$filename" && chown user:user "$FORM_path/$filename"; then
				MESSAGE="File was uploaded successfully:<br/><strong>$filename</strong>"
			else
				ERROR="Error recieving file $filename"
				rm "$FORM_file"
			fi
		fi
	else
		rm "$FORM_file"
	fi
fi

############################
# stuff
############################

freespace=$(df -h $SBC_USERSPACE | grep / | awk {'print $4'})
edit_pathname="$FORM_path/$FORM_edit"
saved_filename="/tmp/.webif/edited-files$edit_pathname"

! empty "$FORM_save" && {
	SAVED=1
	mkdir -p "/tmp/.webif/edited-files$FORM_path"
	echo "$FORM_filecontent" > "$saved_filename"
}

! empty "$FORM_savecommit" && {
	rm -f "$saved_filename"
	MESSAGE="File was saved successfully:<br/><strong>$edit_pathname</strong>"
	echo "$FORM_filecontent" > "$edit_pathname"
}

empty "$FORM_cancel" || FORM_edit=""

############################
# Header
############################
header "Userspace" "Userspace Browser" "Userspace Browser" ''

############################
# main view
############################
if empty "$FORM_edit"; then

	echo "<p>Userspace should be used for general storage / workspace. Shown here is the home directory of user 'user',"
	echo "which is the default login account for SSH. Manage your coding projects, upload files, etc. from this page.</p>"
	echo "<br />"

	(ls -alLe "$FORM_path" 2>/dev/null | sed '/^[^d]/d';
		ls -alLe "$FORM_path" 2>/dev/null | sed '/^[d]/d') 2>/dev/null | awk \
		-v url="$SCRIPT_NAME" \
		-v path="$FORM_path" \
		-f /usr/lib/webif/common.awk \
		-f /usr/lib/webif/browser.awk
		
	
	cat <<EOF
	<div class="settings">
	<form method="POST" name="upload" action="$SCRIPT_NAME" enctype="multipart/form-data" onSubmit="statusupdate()">
	<div class="settings-content">
	<table width="100%" summary="Settings">

		<tr>
			<td>Upload a file:</td>
			<td>
				<input type="file" name="file" />
			</td>
		</tr>
		<tr>
			<td />
			<td>
				<input id="form_submit" type="submit" name="submit" value="Upload" />
				<input type="hidden" name="path" value="$FORM_path" />
				<br />
				<br />
				</form>
			</td>
		</tr>
		<tr>
			<td>Create a directory:</td>
			<td>
				<form method="POST" name="newdir" action="$SCRIPT_NAME">
				<input type="hidden" name="path" value="$FORM_path" />
				<input type="text" name="dirname" />
			</td>
		</tr>
		<tr>
			<td />
			<td>
				<input id="form_submit2" type="submit" name="submit2" value="Create" />
				<br />
				<br />
				</form>
			</td>
		</tr>
		<tr>
			<td colspan="2">Free space remaining on userspace partition: $freespace</td>
		</tr>
	</table>
	</div>
	<div class="settings-help"><blockquote><dl>
		<h4>Notes:</h4>
		<p>Upload file size limit is 4 MB.</p>
	</dl></blockquote></div>
	<div style="clear: both">&nbsp;</div></div>
EOF

else
	exists "$saved_filename" && {
		edit_filename="$saved_filename"
	} || {
		edit_filename="$edit_pathname"
	}
	cat "$edit_filename" 2>/dev/null | awk \
		-v url="$SCRIPT_NAME" \
		-v path="$FORM_path" \
		-v file="$FORM_edit" \
		-f /usr/lib/webif/common.awk \
		-f /usr/lib/webif/editor.awk
fi

footer ?>
<!--
##WEBIF:name:Userspace:100:Userspace Browser
-->
