#!/usr/bin/haserl
<?
. /usr/lib/webif/webif.sh

timeout=45

errormsg()
{
	echo "<font color=\"red\">$1</font>"
	err="true"
}

_board_name="$([ -e /proc/device-tree/model ] && (cat /proc/device-tree/model; echo '';) || (cat /proc/cpuinfo | awk '/^Hardware/ {print $3}'))"
_board_name="$(echo $_board_name | tr '[:upper:]' '[:lower:]')"

# system reboot
if ! equal $FORM_reboot "" ; then
	referer_ip="${HTTP_REFERER#http://}"
	referer_ip="${referer_ip%%/*}"
	if empty "$referer_ip"; then
		router_ip="$(/sbin/ifconfig eth0 | sed -n '/inet addr:/s/ *inet addr:\([[:digit:].]*\) .*/\1/p')"
	else
		router_ip="$referer_ip"
	fi
	if ! empty "$router_ip"; then
		header_inject_head="<meta http-equiv=\"refresh\" content=\"$timeout; url=http://$router_ip/\" />"
	else
		header_inject_head="hm...."
	fi
	
	header "System Recovery / Upgrade"
	
	cat <<EOF
	<br/><br/><br/>
	<table width="90%" border="0" cellpadding="2" cellspacing="2" align="center">
	<tr>
		<td><script type="text/javascript" src="/js/progress.js"></script>Rebooting now...
<br/><br/>
Please wait about $timeout seconds. The PhidgetSBC configuration should automatically reload.
<br/><br/>
<center>
<script type="text/javascript">
<!--
var bar1=createBar(350,15,'white',1,'black','blue',85,7,3,'');
-->
</script>
</center><br/><br/><br/></td>
	</tr>
	</table>
EOF
else
	header "System Recovery / Upgrade"
	
# system upgrade
if ! equal $FORM_upgrade "" ; then

	filename="${FORM_firmware_usb##*/}"
	if [ "$filename" = "system_ubi.img" ]; then
		echo "Confirm: Upgrade rootfs with \"$FORM_firmware_usb\"?"
		upgradetype="rootfs"
	elif [ "$filename" = "uImage" ]; then
		echo "Confirm: Upgrade kernel with \"$FORM_firmware_usb\"?"
		upgradetype="kernel"
	else
		echo "Confirm: Upgrade system with \"$FORM_firmware_usb\"?"
		upgradetype="system"
	fi

	cat <<EOF
	<br />
	<br />
	<form method="post" name="systemupgrade" action="$SCRIPT_NAME">
		<input type="submit" name="back" value="Back" />
		<input type="submit" name="upgrade_confirmed" value="Upgrade" />
		<input type="hidden" name="filename" value="$FORM_firmware_usb" />
		<input type="hidden" name="upgradetype" value="$upgradetype" />
	</form>
EOF

# system upgrade confirmed
elif ! equal $FORM_upgrade_confirmed "" ; then
			
	echo "Upgrading $FORM_upgradetype with file: $FORM_filename ...<br />"
	filesize=$( ls -l $FORM_filename | awk '{print $5}' )
	timeest=$(expr $filesize / 875561)
	
	if [ "$FORM_upgradetype" = "kernel" ]; then
		set_mtd kernel
		if ! flash_eraseall -q $mtd 2>&1 1>/dev/null; then
			errormsg "Error erasing kernel partition.<br />"
		else
			if ! nandwrite -q -p $mtd $FORM_filename 2>&1 1>/dev/null; then
				errormsg "Error flashing kernel partition.<br />"
			fi
		fi
	elif [ "$FORM_upgradetype" = "rootfs" ]; then
		echo "<br />Please wait (~$timeest seconds) ...<br />"
		set_mtd rootfs
		if ! ubiformat $mtd -q -f $FORM_filename 2>&1 1>/dev/null; then
			# If the partition is really corrupted, we have to erase it first. This is preferably avoided as it clobbers UBI erase counters.
			echo "<br />Root partition is corrupted, erasing ...<br />"
			if ! flash_eraseall -q $mtd 2>&1 1>/dev/null; then
				errormsg "Error erasing root partition.<br />"
			else
				echo "Flashing ... <br />"
				if ! ubiformat $mtd -q -f $FORM_filename 2>&1 1>/dev/null; then
					errormsg "Error flashing root partition.<br />"
				fi
			fi
		fi
	elif [ "$FORM_upgradetype" = "system" ]; then
		
		if !(tar -tvf $FORM_filename | grep "system_ubi.img") 2>&1 1>/dev/null; then
			errormsg "<br />Error: $FORM_filename does not contain a system_ubi.img<br />"
		fi
		
		if !(tar -tvf $FORM_filename | grep "uImage") 2>&1 1>/dev/null; then
			errormsg "<br />Error: $FORM_filename does not contain a uImage<br />"
		fi
		
		if [ "$err" != "true" ]; then
			echo "<br />Flashing kernel, Please wait ...<br />"
			set_mtd kernel
			if ! flash_eraseall -q $mtd 2>&1 1>/dev/null; then
				errormsg "Error erasing kernel partition.<br />"
			else
				if ! tar -x -f $FORM_filename uImage -O | nandwrite -q -p $mtd 2>&1 1>/dev/null; then
					errormsg "Error flashing kernel partition.<br />"
				else
					set_mtd rootfs
					ubi_filesize=$( tar -tvf $FORM_filename | grep "system_ubi.img" | awk '{print $3}' )
					timeest=$(expr $ubi_filesize / 875561)
					echo "<br />Flashing rootfs, Please wait (~$timeest seconds) ...<br />"
					if ! tar -x -f $FORM_filename system_ubi.img -O | ubiformat $mtd -q -f - -S ${ubi_filesize} 2>&1 1>/dev/null; then
						# If the partition is really corrupted, we have to erase it first. This is preferably avoided as it clobbers UBI erase counters.
						echo "<br />Root partition is corrupted, erasing ...<br />"
						if ! flash_eraseall -q $mtd 2>&1 1>/dev/null; then
							errormsg "Error erasing root partition.<br />"
						else
							echo "Flashing ... <br />"
							if ! tar -x -f $FORM_filename system_ubi.img -O | ubiformat $mtd -q -f - -S ${ubi_filesize} 2>&1 1>/dev/null; then
								errormsg "Error flashing root partition.<br />"
							fi
						fi
					fi
				fi
			fi
		fi
		
	else
		errormsg "Invalid upgrade type: $FORM_upgradetype<br />"
	fi

	if [ "$err" = "true" ]; then
		echo "<br />Error(s) were encountered, system may be in an inconsistent state!<br /><br />"
	else
		echo "<br />Done.<br /><br />"
	fi
	
	cat <<EOF
	<form method="post" name="reboot" action="$SCRIPT_NAME">
		<input type="submit" name="back" value="Back" />
		<input type="submit" name="reboot" value="Reboot" />
	</form>
EOF

# system reset
elif ! equal $FORM_reset "" ; then

	cat <<EOF
	Confirm: Really reset the system to factory state?
	<br />
	<br />
	<form method="post" name="systemreset" action="$SCRIPT_NAME">
		<input type="submit" name="back" value="Back" />
		<input type="submit" name="reset_confirmed" value="Reset" />
	</form>
EOF

# system reset confirmed
elif ! equal $FORM_reset_confirmed "" ; then

	set_mtd recovery_kernel
	recovery="$mtd"
	set_mtd kernel
	echo "Resetting kernel partition ($mtd) ... <br />"
	if ! flash_eraseall -q $mtd 2>&1 1>/dev/null; then
		errormsg "Error erasing kernel partition.<br />"
	else
		if ! nanddump $recovery -o -b -q | nandwrite -p -q $mtd - 2>&1 1>/dev/null; then
			errormsg "Error flashing kernel partition.<br />"
		else
			set_mtd recovery_fs
			recovery="$mtd"
			set_mtd rootfs

			if equal $_board_name "phidgetsbc2" ; then
				# size of recovery partition - bad eraseblocks!
				bad_blocks=$(nanddump $recovery -o -l 1 2>&1 | grep "Number of bad blocks" | awk '{ print $5 }')
				eraseblock_size=$( mtdinfo $recovery | awk '/Eraseblock size/ { print $3 }' )
				recoveryfs_part_size=$( cat /proc/mtd | awk '/recovery_fs/ {print ("0x"$2)+0}' )
				recoverybad_size=$( expr $bad_blocks "*" $eraseblock_size )
				recoveryfs_size=$( expr $recoveryfs_part_size - $recoverybad_size )
				
				echo "<br />Resetting rootfs partition ($mtd) ... <br />"
				if ! nanddump $recovery -o -b -q | ubiformat $mtd -q -f - -S ${recoveryfs_size} 2>&1 1>/dev/null; then
					# If the partition is really corrupted, we have to erase it first. This is preferably avoided as it clobbers UBI erase counters.
					echo "<br />Root partition is corrupted, erasing ...<br />"
					if ! flash_eraseall -q $mtd 2>&1 1>/dev/null; then
						errormsg "Error erasing root partition.<br />"
					else
						echo "Flashing ... <br />"
						if ! nanddump $recovery -o -b -q | ubiformat $mtd -q -f - -S ${recoveryfs_size} 2>&1 1>/dev/null; then
							errormsg "Error flashing root partition.<br />"
						fi
					fi
				fi
			fi

			# On SBC3, we have no way to recover ubi image size, so we have to just use nandwrite
			if equal $_board_name "phidgetsbc3" ; then
				echo "<br />Resetting rootfs partition ($mtd) ... <br />"
				if ! flash_eraseall -q $mtd 2>&1 1>/dev/null; then
					errormsg "Error erasing root partition.<br />"
				else
					if ! nanddump $recovery -o -b -q | nandwrite -p -q $mtd - 2>&1 1>/dev/null; then
						errormsg "Error flashing root partition.<br />"
					fi
				fi
			fi

		fi		
	fi
	
	if [ "$err" = "true" ]; then
		echo "<br />Error(s) were encountered, system may be in an inconsistent state!<br /><br />"
	else
		echo "<br />Done.<br /><br />"
	fi
	
	cat <<EOF
	<form method="post" name="reboot" action="$SCRIPT_NAME">
		<input type="submit" name="back" value="Back" />
		<input type="submit" name="reboot" value="Reboot" />
	</form>
EOF

# Main page
else
	cat <<EOF
	<form method="post" name="sysrestore" action="$SCRIPT_NAME" enctype="multipart/form-data">
EOF
	
	usb_firmwares=$( (cd /media; find -maxdepth 2) | grep -E "${_board_name}[-_].*\.bin|/uImage|/system_ubi.img" | sed 's/^\./\/media/' )
	
	
	cat <<EOF
	<table width="100%" border="0" cellspacing="0" cellpadding="0">
	<tr>
	  <td colspan="2" class="listtopic">Upgrade System</td>
	</tr>
	<tr>
EOF
	empty "$usb_firmwares" && {
		cat <<EOF
		<td colspan="2" class="settings-help" valign="top"><blockquote><dl>
		<p>No suitable upgrade files were found. Please attach as USB drive containing supported upgrade file(s) and reload this page.</p> 
		<p>Supported files are UBI Images (system_ubi.img), Kernel images (uImage) and Upgrade packages containing both (${_board_name}-*.bin).
		Upgrades must be done from a USB drive. See product manual for more information.</p></dl></blockquote>
		</td>
EOF
	} || {
		cat <<EOF
	  <td class="settings-content" valign="top">
	    <table width="100%" border="0" cellspacing="0" cellpadding="0">
	      <br />
	      <tr>
	        <td width="40%">File: 
	          <select name="firmware_usb">
EOF
		selected=" selected=\"selected\""
		for firmware in $usb_firmwares; do
			filesize=$( ls -lh $firmware | awk '{print $5}' )
			filename=${firmware##*/}
			cat <<EOF
	            <option value="$firmware"$selected>$filename - ($filesize MB)</option>
EOF
			selected=""
		done

		cat <<EOF
	          </select>
	        </td>
	      </tr>
	      <tr>
	        <td>
	          <br />
	          <input type="submit" name="upgrade" value="Upgrade" />
	        </td>
	      </tr>
	    </table>
	  </td>
	  <td class="settings-help" valign="top"><blockquote><dl><p>Upgrade the kernel and/or root filesystem. Supported files are UBI Images (system_ubi.img), 
	  Kernel images (uImage) and Upgrade packages containing both (${_board_name}-*.bin). Upgrades must be done from a USB drive. See product manual for more information.</p></dl></blockquote>
	  </td>
EOF
	}
	cat <<EOF
	</tr>
	</table>
	<br />
EOF

	display_form <<EOF
string|<br />
start_form|Reset System
string|<br />
string|<input type="submit" name="reset" value="Factory Reset" />
helptext|Reset the system to its factory state. This includes re-flashing both the root filesystem and the kernel from backups. All settings/customization will be lost.
end_form
EOF

	display_form <<EOF
string|<br />
start_form|Reboot System
string|<br />
string|<input type="submit" name="reboot" value="Reboot" />
helptext|Reboot back into the standard system.
end_form
EOF

	echo "</form>"
fi
fi

footer
?>
<?
! empty "$FORM_reboot" && {
	reboot &
	exit
}
?>